@@ -193,11 +193,20 @@ void _dbus_real_assert_not_reached (const char *explanation,
 
 #define _DBUS_ZERO(object) (memset (&(object), '\0', sizeof ((object))))
 
+#ifdef offsetof
+#define _DBUS_STRUCT_OFFSET(struct_type, member) \
+    (offsetof (struct_type, member))
+#else
 #define _DBUS_STRUCT_OFFSET(struct_type, member)	\
     ((intptr_t) ((unsigned char*) &((struct_type*) 0)->member))
+#endif
 
+#if defined(__STDC_VERSION__) && __STDC_VERSION__ >= 201112L && !defined(__cplusplus)
+#define _DBUS_ALIGNOF(type) _Alignof(type)
+#else
 #define _DBUS_ALIGNOF(type) \
     (_DBUS_STRUCT_OFFSET (struct { char _1; type _2; }, _2))
+#endif
 
 #if defined(DBUS_DISABLE_CHECKS) || defined(DBUS_DISABLE_ASSERT)
 /* this is an assert and not an error, but in the typical --disable-checks case (you're trying
