@@ -51,7 +51,7 @@ struct DBusServerSocket
 {
   DBusServer base;   /**< Parent class members. */
   int n_fds;         /**< Number of active file handles */
-  int *fds;          /**< File descriptor or -1 if disconnected. */
+  DBusSocket *fds;   /**< File descriptor or DBUS_SOCKET_INVALID if disconnected. */
   DBusWatch **watch; /**< File descriptor watch. */
   char *socket_name; /**< Name of domain socket, to unlink if appropriate */
   DBusNonceFile *noncefile; /**< Nonce file used to authenticate clients */
@@ -75,33 +75,32 @@ socket_finalize (DBusServer *server)
   dbus_free (socket_server->fds);
   dbus_free (socket_server->watch);
   dbus_free (socket_server->socket_name);
-  if (socket_server->noncefile)
-	_dbus_noncefile_delete (socket_server->noncefile, NULL);
-  dbus_free (socket_server->noncefile);
+  _dbus_noncefile_delete (&socket_server->noncefile, NULL);
   dbus_free (server);
 }
 
 /* Return value is just for memory, not other failures. */
 static dbus_bool_t
 handle_new_client_fd_and_unlock (DBusServer *server,
-                                 int         client_fd)
+                                 DBusSocket  client_fd)
 {
   DBusConnection *connection;
   DBusTransport *transport;
   DBusNewConnectionFunction new_connection_function;
   void *new_connection_data;
 
-  _dbus_verbose ("Creating new client connection with fd %d\n", client_fd);
+  _dbus_verbose ("Creating new client connection with fd %" DBUS_SOCKET_FORMAT "\n",
+                 _dbus_socket_printable (client_fd));
 
   HAVE_LOCK_CHECK (server);
 
-  if (!_dbus_set_fd_nonblocking (client_fd, NULL))
+  if (!_dbus_set_socket_nonblocking (client_fd, NULL))
     {
       SERVER_UNLOCK (server);
       return TRUE;
     }
 
-  transport = _dbus_transport_new_for_socket (client_fd, &server->guid_hex, FALSE);
+  transport = _dbus_transport_new_for_socket (client_fd, &server->guid_hex, NULL);
   if (transport == NULL)
     {
       _dbus_close_socket (client_fd, NULL);
@@ -182,25 +181,28 @@ socket_handle_watch (DBusWatch    *watch,
 
   if (flags & DBUS_WATCH_READABLE)
     {
-      int client_fd;
-      int listen_fd;
+      DBusSocket client_fd;
+      DBusSocket listen_fd;
+      int saved_errno;
 
-      listen_fd = dbus_watch_get_socket (watch);
+      listen_fd = _dbus_watch_get_socket (watch);
 
       if (socket_server->noncefile)
           client_fd = _dbus_accept_with_noncefile (listen_fd, socket_server->noncefile);
       else 
           client_fd = _dbus_accept (listen_fd);
 
-      if (client_fd < 0)
+      saved_errno = _dbus_save_socket_errno ();
+
+      if (!_dbus_socket_is_valid (client_fd))
         {
           /* EINTR handled for us */
 
-          if (_dbus_get_is_errno_eagain_or_ewouldblock ())
+          if (_dbus_get_is_errno_eagain_or_ewouldblock (saved_errno))
             _dbus_verbose ("No client available to accept after all\n");
           else
             _dbus_verbose ("Failed to accept a client connection: %s\n",
-                           _dbus_strerror_from_errno ());
+                           _dbus_strerror (saved_errno));
 
           SERVER_UNLOCK (server);
         }
@@ -239,8 +241,11 @@ socket_disconnect (DBusServer *server)
           socket_server->watch[i] = NULL;
         }
 
-      _dbus_close_socket (socket_server->fds[i], NULL);
-      socket_server->fds[i] = -1;
+      if (_dbus_socket_is_valid (socket_server->fds[i]))
+        {
+          _dbus_close_socket (socket_server->fds[i], NULL);
+          _dbus_socket_invalidate (&socket_server->fds[i]);
+        }
     }
 
   if (socket_server->socket_name != NULL)
@@ -273,14 +278,16 @@ static const DBusServerVTable socket_vtable = {
  * @param n_fds number of file descriptors
  * @param address the server's address
  * @param noncefile to be used for authentication (NULL if not needed)
- * @returns the new server, or #NULL if no memory.
+ * @param error location to store reason for failure
+ * @returns the new server, or #NULL on OOM or other error.
  *
  */
 DBusServer*
-_dbus_server_new_for_socket (int              *fds,
+_dbus_server_new_for_socket (DBusSocket       *fds,
                              int               n_fds,
                              const DBusString *address,
-                             DBusNonceFile    *noncefile)
+                             DBusNonceFile    *noncefile,
+                             DBusError        *error)
 {
   DBusServerSocket *socket_server;
   DBusServer *server;
@@ -288,29 +295,29 @@ _dbus_server_new_for_socket (int              *fds,
 
   socket_server = dbus_new0 (DBusServerSocket, 1);
   if (socket_server == NULL)
-    return NULL;
+    goto failed;
 
   socket_server->noncefile = noncefile;
 
-  socket_server->fds = dbus_new (int, n_fds);
+  socket_server->fds = dbus_new (DBusSocket, n_fds);
   if (!socket_server->fds)
-    goto failed_0;
+    goto failed;
 
   socket_server->watch = dbus_new0 (DBusWatch *, n_fds);
   if (!socket_server->watch)
-    goto failed_1;
+    goto failed;
 
   for (i = 0 ; i < n_fds ; i++)
     {
       DBusWatch *watch;
 
-      watch = _dbus_watch_new (fds[i],
+      watch = _dbus_watch_new (_dbus_socket_get_pollable (fds[i]),
                                DBUS_WATCH_READABLE,
                                TRUE,
                                socket_handle_watch, socket_server,
                                NULL);
       if (watch == NULL)
-        goto failed_2;
+        goto failed;
 
       socket_server->n_fds++;
       socket_server->fds[i] = fds[i];
@@ -318,8 +325,9 @@ _dbus_server_new_for_socket (int              *fds,
     }
 
   if (!_dbus_server_init_base (&socket_server->base,
-                               &socket_vtable, address))
-    goto failed_2;
+                               &socket_vtable, address,
+                               error))
+    goto failed;
 
   server = (DBusServer*)socket_server;
 
@@ -331,13 +339,27 @@ _dbus_server_new_for_socket (int              *fds,
                                    socket_server->watch[i]))
         {
           int j;
-          for (j = 0 ; j < i ; j++)
-            _dbus_server_remove_watch (server,
-                                       socket_server->watch[j]);
 
+          /* The caller is still responsible for closing the fds until
+           * we return successfully, so don't let socket_disconnect()
+           * close them */
+          for (j = 0; j < n_fds; j++)
+            _dbus_socket_invalidate (&socket_server->fds[j]);
+
+          /* socket_disconnect() will try to remove all the watches;
+           * make sure it doesn't see the ones that weren't even added
+           * yet */
+          for (j = i; j < n_fds; j++)
+            {
+              _dbus_watch_invalidate (socket_server->watch[j]);
+              _dbus_watch_unref (socket_server->watch[j]);
+              socket_server->watch[j] = NULL;
+            }
+
+          _dbus_server_disconnect_unlocked (server);
           SERVER_UNLOCK (server);
           _dbus_server_finalize_base (&socket_server->base);
-          goto failed_2;
+          goto failed;
         }
     }
 
@@ -346,22 +368,30 @@ _dbus_server_new_for_socket (int              *fds,
   _dbus_server_trace_ref (&socket_server->base, 0, 1, "new_for_socket");
   return (DBusServer*) socket_server;
 
- failed_2:
-  for (i = 0 ; i < n_fds ; i++)
+failed:
+  if (socket_server != NULL)
     {
-      if (socket_server->watch[i] != NULL)
+      if (socket_server->watch != NULL)
         {
-          _dbus_watch_unref (socket_server->watch[i]);
-          socket_server->watch[i] = NULL;
+          for (i = 0; i < n_fds; i++)
+            {
+              if (socket_server->watch[i] != NULL)
+                {
+                  _dbus_watch_invalidate (socket_server->watch[i]);
+                  _dbus_watch_unref (socket_server->watch[i]);
+                  socket_server->watch[i] = NULL;
+                }
+            }
         }
+
+      dbus_free (socket_server->watch);
+      dbus_free (socket_server->fds);
+      dbus_free (socket_server);
     }
-  dbus_free (socket_server->watch);
 
- failed_1:
-  dbus_free (socket_server->fds);
+  if (error != NULL && !dbus_error_is_set (error))
+    _DBUS_SET_OOM (error);
 
- failed_0:
-  dbus_free (socket_server);
   return NULL;
 }
 
@@ -392,28 +422,27 @@ _dbus_server_new_for_tcp_socket (const char     *host,
                                  DBusError      *error,
                                  dbus_bool_t    use_nonce)
 {
-  DBusServer *server;
-  int *listen_fds = NULL;
+  DBusServer *server = NULL;
+  DBusSocket *listen_fds = NULL;
   int nlisten_fds = 0, i;
-  DBusString address;
-  DBusString host_str;
-  DBusString port_str;
-  DBusNonceFile *noncefile;
-  
-  _DBUS_ASSERT_ERROR_IS_CLEAR (error);
+  DBusString address = _DBUS_STRING_INIT_INVALID;
+  DBusString host_str;    /* Initialized as const later, not freed */
+  DBusString port_str = _DBUS_STRING_INIT_INVALID;
+  DBusNonceFile *noncefile = NULL;
+  const char *family_used = NULL;
 
-  noncefile = NULL;
+  _DBUS_ASSERT_ERROR_IS_CLEAR (error);
 
   if (!_dbus_string_init (&address))
     {
       dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
-      return NULL;
+      goto failed;
     }
 
   if (!_dbus_string_init (&port_str))
     {
       dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
-      goto failed_0;
+      goto failed;
     }
 
   if (host == NULL)
@@ -429,11 +458,12 @@ _dbus_server_new_for_tcp_socket (const char     *host,
 
   nlisten_fds =_dbus_listen_tcp_socket (bind, port, family,
                                         &port_str,
+                                        &family_used,
                                         &listen_fds, error);
   if (nlisten_fds <= 0)
     {
       _DBUS_ASSERT_ERROR_IS_SET(error);
-      goto failed_1;
+      goto failed;
     }
 
   _dbus_string_init_const (&host_str, host);
@@ -443,67 +473,52 @@ _dbus_server_new_for_tcp_socket (const char     *host,
       !_dbus_string_append (&address, _dbus_string_get_const_data(&port_str)))
     {
       dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
-      goto failed_2;
+      goto failed;
     }
-  if (family &&
+  if (family_used &&
       (!_dbus_string_append (&address, ",family=") ||
-       !_dbus_string_append (&address, family)))
+       !_dbus_string_append (&address, family_used)))
     {
       dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
-      goto failed_2;
+      goto failed;
     }
 
   if (use_nonce)
     {
-      noncefile = dbus_new0 (DBusNonceFile, 1);
-      if (noncefile == NULL)
-        {
-          dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
-          goto failed_2;
-        }
-
-      if (!_dbus_noncefile_create (noncefile, error))
-          goto failed_3;
+      if (!_dbus_noncefile_create (&noncefile, error))
+        goto failed;
 
       if (!_dbus_string_append (&address, ",noncefile=") ||
           !_dbus_address_append_escaped (&address, _dbus_noncefile_get_path (noncefile)))
         {
           dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
-          goto failed_4;
+          goto failed;
         }
-
     }
 
-  server = _dbus_server_new_for_socket (listen_fds, nlisten_fds, &address, noncefile);
+  server = _dbus_server_new_for_socket (listen_fds, nlisten_fds, &address, noncefile, error);
   if (server == NULL)
-    {
-      dbus_set_error (error, DBUS_ERROR_NO_MEMORY, NULL);
-      goto failed_4;
-    }
+    goto failed;
 
+  /* server has taken ownership of noncefile and the fds in listen_fds */
   _dbus_string_free (&port_str);
   _dbus_string_free (&address);
   dbus_free(listen_fds);
 
   return server;
 
- failed_4:
-  _dbus_noncefile_delete (noncefile, NULL);
+failed:
+  _dbus_noncefile_delete (&noncefile, NULL);
 
- failed_3:
-  dbus_free (noncefile);
+  if (listen_fds != NULL)
+    {
+      for (i = 0; i < nlisten_fds; i++)
+        _dbus_close_socket (listen_fds[i], NULL);
+      dbus_free (listen_fds);
+    }
 
- failed_2:
-  for (i = 0 ; i < nlisten_fds ; i++)
-    _dbus_close_socket (listen_fds[i], NULL);
-  dbus_free(listen_fds);
-
- failed_1:
   _dbus_string_free (&port_str);
-
- failed_0:
   _dbus_string_free (&address);
-
   return NULL;
 }
 
@@ -583,4 +598,3 @@ _dbus_server_socket_own_filename (DBusServer *server,
 
 
 /** @} */
-
